@@ -6,7 +6,6 @@
 
 #include <sys/types.h>
 #include <sys/stat.h>
-#include <pwd.h>
 #include <unistd.h>
 
 #include <arc/loader/Loader.h>
@@ -23,7 +22,6 @@
 #include <arc/FileUtils.h>
 #include <arc/Utils.h>
 #include <arc/otokens/openid_metadata.h>
-#include "job.h"
 #include "grid-manager/log/JobLog.h"
 #include "grid-manager/log/JobsMetrics.h"
 #include "grid-manager/log/HeartBeatMetrics.h"
@@ -69,6 +67,7 @@ char const* ARexService::LogsPath = "*logs";
 char const* ARexService::NewPath = "*new";
 char const* ARexService::DelegationPath = "*deleg";
 char const* ARexService::CachePath = "cache";
+char const* ARexService::RestPath = "rest";
 
 #define AREX_POLICY_OPERATION_URN "http://www.nordugrid.org/schemas/policy-arc/types/a-rex/operation"
 #define AREX_POLICY_OPERATION_ADMIN "Admin"
@@ -313,13 +312,6 @@ static Arc::Plugin* get_service(Arc::PluginArgument* arg) {
     return arex;
 }
 
-class ARexConfigContext:public Arc::MessageContextElement, public ARexGMConfig {
- public:
-  ARexConfigContext(GMConfig& config,const std::string& uname,const std::string& grid_name,const std::string& service_endpoint):
-    ARexGMConfig(config,uname,grid_name,service_endpoint) { };
-  virtual ~ARexConfigContext(void) { };
-};
-
 void CountedResource::Acquire(void) {
   lock_.lock();
   while((limit_ >= 0) && (count_ >= limit_)) {
@@ -435,118 +427,6 @@ Arc::MCC_Status ARexService::make_empty_response(Arc::Message& outmsg) {
   return Arc::MCC_Status(Arc::STATUS_OK);
 }
 
-ARexGMConfig* ARexService::get_configuration(Arc::Message& inmsg) {
-  Arc::MessageContextElement* mcontext = (*inmsg.Context())["arex.gmconfig"];
-  if(mcontext) {
-    try {
-      ARexConfigContext* config = dynamic_cast<ARexConfigContext*>(mcontext);
-      if(config) {
-        logger_.msg(Arc::DEBUG,"Using cached local account '%s'", config->User().Name());
-        return config;
-      }
-    } catch(std::exception& e) { };
-  };
-  // TODO: do configuration detection
-  // TODO: do mapping to local unix name
-  std::string uname;
-  uname=inmsg.Attributes()->get("SEC:LOCALID");
-  if(uname.empty()) uname=uname_;
-  if(uname.empty()) {
-    if(getuid() == 0) {
-      logger_.msg(Arc::ERROR, "Will not map to 'root' account by default");
-      return NULL;
-    };
-    struct passwd pwbuf;
-    char buf[4096];
-    struct passwd* pw;
-    if(getpwuid_r(getuid(),&pwbuf,buf,sizeof(buf),&pw) == 0) {
-      if(pw && pw->pw_name) {
-        uname = pw->pw_name;
-      };
-    };
-  };
-  if(uname.empty()) {
-    logger_.msg(Arc::ERROR, "No local account name specified");
-    return NULL;
-  };
-  logger_.msg(Arc::DEBUG,"Using local account '%s'",uname);
-  std::string grid_name = inmsg.Attributes()->get("TLS:IDENTITYDN");
-  if(grid_name.empty()) {
-    // Try tokens if TLS has no information about user identity
-    logger_.msg(Arc::ERROR, "TLS provides no identity, going for OTokens");
-    grid_name = inmsg.Attributes()->get("OTOKENS:IDENTITYDN");
-    /*
-    Below is an example on how obtained token can be exchanged.
-
-    Arc::SecAttr* sattr = inmsg.Auth()->get("OTOKENS");    
-    if(!sattr) sattr = inmsg.AuthContext()->get("OTOKENS");
-    if(sattr) {
-      std::string token = sattr->get("");
-      if(!token.empty()) {
-        Arc::OpenIDMetadata tokenMetadata;
-        Arc::OpenIDMetadataFetcher metaFetcher(sattr->get("iss").c_str());
-        if(metaFetcher.Fetch(tokenMetadata)) {
-          char const * tokenEndpointUrl = tokenMetadata.TokenEndpoint();
-          if(tokenEndpointUrl) {
-            Arc::OpenIDTokenFetcher tokenFetcher(tokenEndpointUrl,
-                  "c85e84e8-c9ea-4ecc-8123-070df2c10e0e",
-                  "dRnakcoaT-9YA6T1LzeLAqeEu7jLBxeTWFyQMbJ6BWZonjEcE060-dn8EWAfpZmPq3x7oTjUnu6mamYylBaNhw");
-
-            std::list<std::string> scopes;
-            scopes.push_back("storage.read:/");
-            scopes.push_back("storage.create:/");
-            std::list<std::string> audiences;
-            audiences.push_back("se1.example");
-            audiences.push_back("se2.example");
-
-            Arc::OpenIDTokenFetcher::TokenList tokens;
-            if(tokenFetcher.Fetch("urn:ietf:params:oauth:grant-type:token-exchange", token, scopes, audiences, tokens)) {
-              for(auto const & token : tokens) {
-                logger_.msg(Arc::ERROR, "Token response: %s : %s", token.first, token.second);
-              };
-            } else logger_.msg(Arc::ERROR, "Failed to fetch token");
-          } else logger_.msg(Arc::ERROR, "Token metadata contains no token endpoint");;
-        } else logger_.msg(Arc::ERROR, "Failed to fetch token metadata");
-      } else logger_.msg(Arc::ERROR, "There is no token in sec attr");
-    } else logger_.msg(Arc::ERROR, "There is no otoken sec attr");
-    */
-  };
-  std::string endpoint = endpoint_;
-  if(endpoint.empty()) {
-    std::string http_endpoint = inmsg.Attributes()->get("HTTP:ENDPOINT");
-    std::string tcp_endpoint = inmsg.Attributes()->get("TCP:ENDPOINT");
-    bool https_proto = !grid_name.empty();
-    endpoint = tcp_endpoint;
-    if(https_proto) {
-      endpoint="https"+endpoint;
-    } else {
-      endpoint="http"+endpoint;
-    };
-    endpoint+=GetPath(http_endpoint);
-  };
-
-  // Do authorization according to allowed/denied authgroups
-  std::list<std::pair<bool,std::string> > const & groups = config_.MatchingGroups("");
-  if(!groups.empty()) {
-    if(!match_groups(groups, inmsg)) {
-      logger_.msg(Arc::ERROR, "Service access is not allowed for this user");
-      return NULL;
-    };
-  };
-
-  // Create configuration for this user
-  ARexConfigContext* config=new ARexConfigContext(config_,uname,grid_name,endpoint);
-  if(config) {
-    if(*config) {
-      inmsg.Context()->Add("arex.gmconfig",config);
-      return config;
-    };
-    delete config; config=NULL;
-    logger_.msg(Arc::ERROR, "Failed to acquire A-REX's configuration");
-  };
-  return NULL;
-}
-
 static std::string GetPath(Arc::Message &inmsg,std::string &base) {
   base = inmsg.Attributes()->get("HTTP:ENDPOINT");
   Arc::AttributeIterator iterator = inmsg.Attributes()->getAll("PLEXER:EXTENSION");
@@ -577,6 +457,47 @@ static void GetIdFromPath(std::string& subpath, std::string& id) {
   while(subpath[0] == '/') subpath.erase(0,1);
 }
 
+Arc::MCC_Status ARexService::preProcessSecurity(Arc::Message& inmsg,Arc::Message& outmsg,Arc::SecAttr* sattr,bool is_soap,ARexConfigContext*& config) {
+  config = NULL;
+  if(sattr) inmsg.Auth()->set("AREX",sattr);
+
+  {
+    Arc::MCC_Status sret = ProcessSecHandlers(inmsg,"incoming");
+    if(!sret) {
+      logger_.msg(Arc::ERROR, "Security Handlers processing failed: %s", std::string(sret));
+      std::string fault_str = "Not authorized: " + std::string(sret);
+      return is_soap ?
+        make_soap_fault(outmsg, fault_str.c_str()) :
+        make_http_fault(outmsg, HTTP_ERR_FORBIDDEN, fault_str.c_str());
+    };
+  };
+
+  // Process grid-manager configuration if not done yet
+  config = ARexConfigContext::GetRutimeConfiguration(inmsg, config_, uname_, endpoint_);
+  if(!config) {
+    logger_.msg(Arc::ERROR, "Can't obtain configuration");
+    // Service is not operational
+    char const* fault = "User can't be assigned configuration";
+    return is_soap ?
+      make_soap_fault(outmsg, fault) :
+      make_http_fault(outmsg, HTTP_ERR_FORBIDDEN, fault);
+  };
+  config->ClearAuths();
+  config->AddAuth(inmsg.Auth());
+  config->AddAuth(inmsg.AuthContext());
+
+  return Arc::MCC_Status(Arc::STATUS_OK);
+}
+
+Arc::MCC_Status ARexService::postProcessSecurity(Arc::Message& outmsg) {
+  Arc::MCC_Status sret = ProcessSecHandlers(outmsg,"outgoing");
+  if(!sret) {
+    logger_.msg(Arc::ERROR, "Security Handlers processing failed: %s", std::string(sret));
+    delete outmsg.Payload(NULL);
+  };
+  return sret;
+}
+
 Arc::MCC_Status ARexService::process(Arc::Message& inmsg,Arc::Message& outmsg) {
   // Split request path into parts: service, job and file path. 
   // TODO: make it HTTP independent
@@ -591,7 +512,7 @@ Arc::MCC_Status ARexService::process(Arc::Message& inmsg,Arc::Message& outmsg) {
   std::string id;
   GetIdFromPath(subpath, id);
   // Make SubOpCache separate service
-  enum { SubOpNone, SubOpInfo, SubOpLogs, SubOpNew, SubOpDelegation, SubOpCache } sub_op = SubOpNone;
+  enum { SubOpNone, SubOpInfo, SubOpLogs, SubOpNew, SubOpDelegation, SubOpCache, SubOpRest } sub_op = SubOpNone;
   // Sort out path
   if(id == InfoPath) {
     sub_op = SubOpInfo;
@@ -608,6 +529,9 @@ Arc::MCC_Status ARexService::process(Arc::Message& inmsg,Arc::Message& outmsg) {
   } else if(id == CachePath) {
     sub_op = SubOpCache;
     id.erase();
+  } else if(id == RestPath) {
+    sub_op = SubOpRest;
+    id.erase();
   };
   logger_.msg(Arc::VERBOSE, "process: id: %s", id);
   logger_.msg(Arc::VERBOSE, "process: subop: %s", (sub_op==SubOpNone)?"none":
@@ -615,9 +539,27 @@ Arc::MCC_Status ARexService::process(Arc::Message& inmsg,Arc::Message& outmsg) {
                                                   ((sub_op==SubOpLogs)?LogsPath:
                                                   ((sub_op==SubOpNew)?NewPath:
                                                   ((sub_op==SubOpDelegation)?DelegationPath:
-                                                  ((sub_op==SubOpCache)?CachePath:"unknown"))))));
+                                                  ((sub_op==SubOpCache)?CachePath:
+                                                  ((sub_op==SubOpRest)?RestPath:"unknown")))))));
   logger_.msg(Arc::VERBOSE, "process: subpath: %s", subpath);
 
+  // Switch to REST ASAP
+  if(sub_op == SubOpRest) {
+    ARexSecAttr* sattr = new ARexSecAttr(std::string(JOB_POLICY_OPERATION_UNDEFINED));
+    if(sattr) sattr->SetResource(endpoint,id,subpath);
+    ARexConfigContext* config(NULL);
+    Arc::MCC_Status sret = preProcessSecurity(inmsg,outmsg,sattr,false,config);
+    if(!sret) return sret;
+
+    sret = rest_.process(inmsg, outmsg);
+
+    if(sret) {
+      sret = postProcessSecurity(outmsg);
+    }
+
+    return sret;
+  }
+
   // Sort out request and identify operation requested
   Arc::PayloadSOAP* inpayload = NULL;
   Arc::XMLNode op;
@@ -656,51 +598,10 @@ Arc::MCC_Status ARexService::process(Arc::Message& inmsg,Arc::Message& outmsg) {
   } else if(method == "DELETE") {
     sattr = new ARexSecAttr(std::string(JOB_POLICY_OPERATION_MODIFY));
   }
-  if(sattr) {
-    inmsg.Auth()->set("AREX",sattr);
-    sattr->SetResource(endpoint,id,subpath);
-  }
-
-  {
-    Arc::MCC_Status sret = ProcessSecHandlers(inmsg,"incoming");
-    if(!sret) {
-      logger_.msg(Arc::ERROR, "Security Handlers processing failed: %s", std::string(sret));
-      std::string fault_str = "Not authorized: " + std::string(sret);
-      return (method == "POST") ?
-        make_soap_fault(outmsg, fault_str.c_str()) :
-        make_http_fault(outmsg, HTTP_ERR_FORBIDDEN, fault_str.c_str());
-    };
-  };
-
-  // Process grid-manager configuration if not done yet
-  ARexGMConfig* config = get_configuration(inmsg);
-  if(!config) {
-    // Service is not operational except public information.
-    // But public information also has own authorization rules
-    if(!config_.PublicInformationEnabled()) {
-      logger_.msg(Arc::VERBOSE, "Can't obtain configuration. Public information is disabled.");
-      char const* fault = "User can't be assigned configuration";
-      return (method == "POST") ?
-        make_soap_fault(outmsg, fault) :
-        make_http_fault(outmsg, HTTP_ERR_FORBIDDEN, fault);
-    };
-    // Check additional authorization rules
-    std::list<std::pair<bool,std::string> > const & groups = config_.MatchingGroupsPublicInformation();
-    if(!groups.empty()) {
-      if(!match_groups(groups, inmsg)) {
-        logger_.msg(Arc::VERBOSE, "Can't obtain configuration. Public information is disallowed for this user.");
-        char const* fault = "User can't be assigned configuration";
-        return (method == "POST") ?
-          make_soap_fault(outmsg, fault) :
-          make_http_fault(outmsg, HTTP_ERR_FORBIDDEN, fault);
-      };
-    };
-    logger_.msg(Arc::VERBOSE, "Can't obtain configuration. Only public information is provided.");
-  } else {
-    config->ClearAuths();
-    config->AddAuth(inmsg.Auth());
-    config->AddAuth(inmsg.AuthContext());
-  };
+  if(sattr) sattr->SetResource(endpoint,id,subpath);
+  ARexConfigContext* config(NULL);
+  Arc::MCC_Status sret = preProcessSecurity(inmsg,outmsg,sattr,method=="POST",config);
+  if(!sret) return sret;
 
   // Identify which of served endpoints request is for.
   // Using simplified algorithm - POST for SOAP messages,
@@ -713,16 +614,7 @@ Arc::MCC_Status ARexService::process(Arc::Message& inmsg,Arc::Message& outmsg) {
       logger_.msg(Arc::ERROR, "POST request on special path is not supported");
       return make_soap_fault(outmsg);
     };
-    if (!id.empty()) {
-      // Listing operations for session directories
-      // TODO: proper failure like interface is not supported
-      logger_.msg(Arc::ERROR, "Per-job POST/SOAP requests are not supported");
-      return make_soap_fault(outmsg,"Operation not supported");
-    };
-    {
-      // TODO: Split this complex code into separate methods. Make handling of anonymous (config == null) 
-      // requests processing cleaner.
-
+    if(id.empty()) {
       // Factory operations
       logger_.msg(Arc::VERBOSE, "process: factory endpoint");
       Arc::PayloadSOAP* outpayload = new Arc::PayloadSOAP(ns_);
@@ -731,7 +623,6 @@ Arc::MCC_Status ARexService::process(Arc::Message& inmsg,Arc::Message& outmsg) {
       outpayload->Namespaces(ns_);
       if(config_.EMIESInterfaceEnabled() && MatchXMLNamespace(op,ES_CREATE_NAMESPACE)) {
         // Aplying known namespaces
-        if(!config) return make_soap_fault(outmsg, "User can't be assigned configuration");
         inpayload->Namespaces(ns_);
         if(MatchXMLName(op,"CreateActivity")) {
           CountedResourceLock cl_lock(beslimit_);
@@ -741,7 +632,6 @@ Arc::MCC_Status ARexService::process(Arc::Message& inmsg,Arc::Message& outmsg) {
         }
       } else if(config_.EMIESInterfaceEnabled() && MatchXMLNamespace(op,ES_RINFO_NAMESPACE)) {
         // Aplying known namespaces
-        // Resource information is available for anonymous requests - null config is handled properly
         inpayload->Namespaces(ns_);
         if(MatchXMLName(op,"GetResourceInfo")) {
           CountedResourceLock cl_lock(infolimit_);
@@ -753,7 +643,6 @@ Arc::MCC_Status ARexService::process(Arc::Message& inmsg,Arc::Message& outmsg) {
           SOAP_NOT_SUPPORTED;
         }
       } else if(config_.EMIESInterfaceEnabled() && MatchXMLNamespace(op,ES_MANAG_NAMESPACE)) {
-        if(!config) return make_soap_fault(outmsg, "User can't be assigned configuration");
         // Aplying known namespaces
         inpayload->Namespaces(ns_);
         if(MatchXMLName(op,"PauseActivity")) {
@@ -784,7 +673,6 @@ Arc::MCC_Status ARexService::process(Arc::Message& inmsg,Arc::Message& outmsg) {
           SOAP_NOT_SUPPORTED;
         }
       } else if(config_.EMIESInterfaceEnabled() && MatchXMLNamespace(op,ES_AINFO_NAMESPACE)) {
-        if(!config) return make_soap_fault(outmsg, "User can't be assigned configuration");
         // Aplying known namespaces
         inpayload->Namespaces(ns_);
         if(MatchXMLName(op,"ListActivities")) {
@@ -800,7 +688,6 @@ Arc::MCC_Status ARexService::process(Arc::Message& inmsg,Arc::Message& outmsg) {
           SOAP_NOT_SUPPORTED;
         }
       } else if(config_.ARCInterfaceEnabled() && MatchXMLNamespace(op,BES_ARC_NAMESPACE)) {
-        if(!config) return make_soap_fault(outmsg, "User can't be assigned configuration");
         // Aplying known namespaces
         inpayload->Namespaces(ns_);
         if(MatchXMLName(op,"CacheCheck")) {
@@ -809,7 +696,6 @@ Arc::MCC_Status ARexService::process(Arc::Message& inmsg,Arc::Message& outmsg) {
           SOAP_NOT_SUPPORTED;
         }
       } else if(delegation_stores_.MatchNamespace(*inpayload)) {
-        if(!config) return make_soap_fault(outmsg, "User can't be assigned configuration");
         // Aplying known namespaces
         inpayload->Namespaces(ns_);
         CountedResourceLock cl_lock(beslimit_);
@@ -870,13 +756,14 @@ Arc::MCC_Status ARexService::process(Arc::Message& inmsg,Arc::Message& outmsg) {
         logger_.msg(Arc::VERBOSE, "process: response=%s",str);
       };
       outmsg.Payload(outpayload);
+    } else {
+      // Listing operations for session directories
+      // TODO: proper failure like interface is not supported
+      logger_.msg(Arc::ERROR, "Per-job POST/SOAP requests are not supported");
+      return make_soap_fault(outmsg,"Operation not supported");
     };
-    Arc::MCC_Status sret = ProcessSecHandlers(outmsg,"outgoing");
-    if(!sret) {
-      logger_.msg(Arc::ERROR, "Security Handlers processing failed: %s", std::string(sret));
-      delete outmsg.Payload(NULL);
-      return sret;
-    };
+    Arc::MCC_Status sret = postProcessSecurity(outmsg);
+    if(!sret) return sret;
     return Arc::MCC_Status(Arc::STATUS_OK);
   } else if(method == "GET") {
     // HTTP plugin either provides buffer or stream
@@ -906,12 +793,8 @@ Arc::MCC_Status ARexService::process(Arc::Message& inmsg,Arc::Message& outmsg) {
         break;
     };
     if(ret) {
-      Arc::MCC_Status sret = ProcessSecHandlers(outmsg,"outgoing");
-      if(!sret) {
-        logger_.msg(Arc::ERROR, "Security Handlers processing failed: %s",std::string(sret));
-        delete outmsg.Payload(NULL);
-        return sret;
-      };
+      Arc::MCC_Status sret = postProcessSecurity(outmsg);
+      if(!sret) return sret;
     };
     return ret;
   } else if(method == "HEAD") {
@@ -941,12 +824,8 @@ Arc::MCC_Status ARexService::process(Arc::Message& inmsg,Arc::Message& outmsg) {
         break;
     };
     if(ret) {
-      Arc::MCC_Status sret = ProcessSecHandlers(outmsg,"outgoing");
-      if(!sret) {
-        logger_.msg(Arc::ERROR, "Security Handlers processing failed: %s", std::string(sret));
-        delete outmsg.Payload(NULL);
-        return sret;
-      };
+      Arc::MCC_Status sret = postProcessSecurity(outmsg);
+      if(!sret) return sret;
     };
     return ret;
   } else if(method == "PUT") {
@@ -975,12 +854,8 @@ Arc::MCC_Status ARexService::process(Arc::Message& inmsg,Arc::Message& outmsg) {
         break;
     };
     if(ret) {
-      Arc::MCC_Status sret = ProcessSecHandlers(outmsg,"outgoing");
-      if(!sret) {
-        logger_.msg(Arc::ERROR, "Security Handlers processing failed: %s", std::string(sret));
-        delete outmsg.Payload(NULL);
-        return sret;
-      };
+      Arc::MCC_Status sret = postProcessSecurity(outmsg);
+      if(!sret) return sret;
     };
     return ret;
   } else if(method == "DELETE") {
@@ -1009,12 +884,8 @@ Arc::MCC_Status ARexService::process(Arc::Message& inmsg,Arc::Message& outmsg) {
         break;
     };
     if(ret) {
-      Arc::MCC_Status sret = ProcessSecHandlers(outmsg,"outgoing");
-      if(!sret) {
-        logger_.msg(Arc::ERROR, "Security Handlers processing failed: %s", std::string(sret));
-        delete outmsg.Payload(NULL);
-        return sret;
-      };
+      Arc::MCC_Status sret = postProcessSecurity(outmsg);
+      if(!sret) return sret;
     };
     return ret;
   } else if(!method.empty()) {
@@ -1169,7 +1040,8 @@ ARexService::ARexService(Arc::Config *cfg,Arc::PluginArgument *parg):Arc::Servic
               infodoc_(true),
               infoprovider_wakeup_period_(0),
               all_jobs_count_(0),
-              gm_(NULL) {
+              gm_(NULL),
+              rest_(cfg, parg, config_, delegation_stores_, all_jobs_count_) {
   valid = false;
   config_.SetJobLog(new JobLog());
   config_.SetJobsMetrics(new JobsMetrics());
