@@ -291,6 +291,13 @@ sub collect($) {
             # merge cluster wide and queue-specific options
             my $sconfig = { %{$config->{service}}, %{$config->{shares}{$share}} };
 
+            my @queue_advertisedvos = ();
+            if ($sconfig->{AdvertisedVO}) {
+                @queue_advertisedvos = @{$sconfig->{AdvertisedVO}};
+                # add VO: suffix to each advertised VO
+                @queue_advertisedvos = map { "VO:".$_ } @queue_advertisedvos;
+            }
+
             $sconfig->{ExecutionEnvironmentName} ||= [];
             my @nxenvs = @{$sconfig->{ExecutionEnvironmentName}};
 
@@ -374,6 +381,8 @@ sub collect($) {
                 $q->{totalcpus} = $qinfo->{totalcpus};
             }	
 
+            $q->{acl} = [ @queue_advertisedvos ] if @queue_advertisedvos;
+
             keys %$gmjobs_info; # reset iterator of each()
 
             # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
